==> helpdesk.ron.cz?menuid=download&filename=ronportal_4_2107_155.zip: ronportal/ronportal/kapacity/sadapodminekplanovani.php <==
<?php
/**
 * stanislav.elbl, 10.03.2021
 */

namespace RonPortal\Kapacity;

use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\StringUtils;
use RonPortal\RonPortalDBRecord;
use RonPortal\Rozdeleni;

class SadaPodminekPlanovani
{
    public $id;

    public $datumOd;
    public $datumDo;

    /** @var int    0=vychozi, 1=prioritni, 2=trvala  */
    public $prioritni;

    /** @var int    0=bez omezeni, 1=typ dne, 2=dny v tydnu */
    public $typOmezeni;
    /** @var int    typ dne podle kalendare u daneho strediska */
    public $typDne;
    /** @var int    Bitova maska - 1=Po,2=Ut,...,64=Ne */
    public $denVTydnu;

    /** @var PodminkaPlanovaniKapacit[]  Seznam podminek */
    public $podminky = [];

    /**
     * Nacte sady podminek platne pro zadane rozdeleni
     * @param Database $database
     * @param int $typRozdeleni
     * @param string $rozdeleni
     * @param int $obdobiOd
     * @param int $obdobiDo
     * @param string $filter
     * @return SadaPodminekPlanovani[]
     * @throws \Ron\DB\DBException
     */
    public static function nactiPodminky($database, $typRozdeleni, $rozdeleni, $obdobiOd, $obdobiDo, $filter = null)
    {
        $result = [];

        $params = $database ->createParameters();

        $sql =
            'SELECT SadaPodminekRozdeleni.IdSady, SadaPodminekRozdeleni.DatumOd, SadaPodminekRozdeleni.DatumDo, SadaPodminekRozdeleni.Prioritni,' . // 0..3
            ' SadaPodminekRozdeleni.TypOmezeni, SadaPodminekRozdeleni.TypDne, SadaPodminekRozdeleni.DenVTydnu,' . // 4..6
            ' PodminkaPlanovaniKapacit.Id,' .    // 7
            ' PodminkaPlanovaniKapacit.TypPodminky, PodminkaPlanovaniKapacit.Smena, PodminkaPlanovaniKapacit.CasOd, PodminkaPlanovaniKapacit.CasDo,' . // 8..11
            ' PodminkaPlanovaniKapacit.KodTypuRozdeleni, PodminkaPlanovaniKapacit.Rozdeleni,' . // 12,13
            ' PodminkaPlanovaniKapacit.KodDovednosti, PodminkaPlanovaniKapacit.KodUrovne,' . // 14,15
            ' PodminkaPlanovaniKapacit.PocetMin, PodminkaPlanovaniKapacit.PocetMax, PodminkaPlanovaniKapacit.TypKontroly,' . // 16..18
            ' PodminkaPlanovaniKapacit.SlotProPlanovani, PodminkaPlanovaniKapacit.LzeEditovat,' . // 19,20
            ' PodminkaPlanovaniKapacit.Update_UserName, PodminkaPlanovaniKapacit.Update_Timestamp' . // 21,22
            "\n" .
            'FROM SadaPodminekRozdeleni LEFT OUTER JOIN PodminkaPlanovaniKapacit' .
            '  ON PodminkaPlanovaniKapacit.IdSady = SadaPodminekRozdeleni.IdSady' .
            "\n" .
            'WHERE SadaPodminekRozdeleni.KodTypuRozdeleni = ' . $params ->addParameter($typRozdeleni) .
            '  AND SadaPodminekRozdeleni.Rozdeleni = ' . $params ->addParameter($rozdeleni) .
            '  AND SadaPodminekRozdeleni.DatumOd <= ' . $params ->addParameter($database ->dateToParam($obdobiDo)) .
            '  AND (SadaPodminekRozdeleni.DatumDo IS NULL OR SadaPodminekRozdeleni.DatumDo >= ' . $params ->addParameter($database ->dateToParam($obdobiOd)) . ')';

        if ($filter) {
            $sql .= ' AND ' . $filter;
        }

        $sql .=
            ' ORDER BY SadaPodminekRozdeleni.IdSady, SadaPodminekRozdeleni.DatumOd, PodminkaPlanovaniKapacit.CasOd';

        $cursor = $database ->getSqlResult($sql, $params);

        /** @var SadaPodminekPlanovani  $sada */
        $sada = null;

        while ($row = $cursor ->fetch()) {
            $datumOd = $database ->stringToTimestamp($row[1]);
            if (!$sada  ||  $row[0] != $sada ->id  ||  $datumOd != $sada ->datumOd) {
                $sada = new SadaPodminekPlanovani();
                $sada ->id = $row[0];
                $sada ->datumOd = $datumOd;
                $sada ->datumDo = $database ->stringToTimestamp($row[2]);
                $sada ->prioritni = (int)$row[3];
                $sada ->typOmezeni = (int)$row[4];
                $sada ->typDne = (int)$row[5];
                $sada ->denVTydnu = $row[6];
                $result[] = $sada;
            }

            if (isset($row[7])) {
                $podminka = new PodminkaPlanovaniKapacit();
                $podminka->id = (int)$row[7];
                $podminka->typPodminky = (int)$row[8];
                if ($podminka->typPodminky == 0) {
                    // smena
                    $podminka->smena = $row[9];
                }
                $podminka->casOd = $database->stringToTimeOfDay($row[10]);
                $podminka->casDo = $database->stringToTimeOfDay($row[11]);

                $podminka->pocetMin = (int)$row[16];
                $podminka->pocetMax = (int)$row[17];

                $podminka->kodTypuRozdeleni = ($row[12] && $row[13]) ? (int)$row[12] : null;
                $podminka->rozdeleni = ($row[12] && $row[13]) ? $row[13] : null;

                $podminka->kodDovednosti = $row[14];
                $podminka->kodUrovne = isset($row[15]) ? (int)$row[15] : null;

                $podminka->typKontroly = $row[18];

                $podminka->slotProPlanovani = $row[19];
                $podminka->lzeEditovat = $row[20];

                $podminka->updateUserName = $row[21];
                $podminka->updateTimestamp = $database ->stringToTimestamp($row[22]);

                $sada->podminky[] = $podminka;
            }
        }

        $cursor ->close();

        // TODO: pro podmiky typu "Operace dochazky" nacist prirazene operace

        return $result;
    }


    /**
     * Nacte sady podminek platne pro zadane rozdeleni a den. Vraci jen sady, ktere se maji skutecne pouzit,
     * takze pokud existuje prioritni sada, tak nevraci vychozi.
     * @param Database $database
     * @param int $typRozdeleni
     * @param string $rozdeleni
     * @param int $datum
     * @param string $filter
     * @return SadaPodminekPlanovani[]
     * @throws \Ron\DB\DBException
     */
    public static function nactiSadyPodminekProDen($database, $typRozdeleni, $rozdeleni, $datum, $filter = null)
    {
        $sadyPodminek = self::nactiPodminky($database, $typRozdeleni, $rozdeleni, $datum, $datum, $filter);

        // profiltruje jen platne sady - vynecha vychozi, pokud existuje prioritni
        $pouziteSady = [];
        $vychoziSada = null;
        $jePrioritni = false;
        foreach ($sadyPodminek as $sada) {
            if ($sada ->jePlatnaProDen($datum)) {
                if ($sada ->prioritni === 1) {
                    // prioritni
                    $pouziteSady[] = $sada;
                    $jePrioritni = true;
                } else if ($sada ->prioritni === 2) {
                    // trvala
                    $pouziteSady[] = $sada;
                } else {
                    // vychozi - pouzije se jen v pripade, ze neexistuje prioritni
                    $vychoziSada = $sada;
                }
            }
        }

        if (!$jePrioritni  &&  $vychoziSada) {
            $pouziteSady[] = $vychoziSada;
        }

        return $pouziteSady;
    }


    /**
     * @return array
     */
    public function getJson()
    {
        $result = [
            'id' => $this ->id,
            'datumOdStr' => StringUtils::jsonDate($this ->datumOd),
            'datumDoStr' => StringUtils::jsonDate($this ->datumDo),
            'prioritni' => $this ->prioritni ? 1 : 0,
            'typOmezeni' => $this ->typOmezeni,
            'typDne' => $this ->typDne,
            'denVTydnu' => $this ->denVTydnu,
            'podminky' => []
        ];

        foreach ($this ->podminky as $podminka) {
            $result['podminky'][] = $podminka ->getJson();
        }

        return $result;
    }


    /**
     * Vraci true, pokud je sada platna pro zadane datum (podle datumu platnosti a omezeni na typ dne nebo den v tydnu)
     * @param $datum
     * @param int $typDne
     * @return bool|int
     */
    public function jePlatnaProDen($datum, $typDne = 0)
    {
        // kontrola platnosti od do
        if ($datum < $this ->datumOd) return false;
        if ($this ->datumDo  &&  $datum > $this ->datumDo) return false;

        // kontrola podle typu dne
        if ($this ->typOmezeni === 1  &&  $this ->typDne != $typDne) {
            return false;
        }

        // kontrola podle dnu v tydnu
        if ($this ->typOmezeni === 2) {
            // jen vybrane dny v tydnu. Bit 1=Po atd.
            return (0x01 << (date('N', $datum) - 1)) & $this ->denVTydnu;
        }

        // zadne omezeni
        return true;
    }


    public function najdiPodminku($hledanaPodminka)
    {
        foreach ($this ->podminky as $podminka) {
            if ($podminka ->isSameAs($hledanaPodminka)) {
                return $podminka;
            }
        }

        return null;
    }


    /**
     * Vytvori novou sadu podminek a priradi ji aktualnimu rozdeleni. Vraci Id nove vytvoreneho zaznamu.
     * @param Database $database
     * @param Rozdeleni $rozdeleni
     * @param int $datum
     * @param object|string $logDescription
     * @return int
     * @throws DBException|\Exception
     */
    public static function vytvorSaduPodminekProDen($database, $rozdeleni, $datum, $logDescription)
    {
        // vytvori sadu podminek
        $rec = new RonPortalDBRecord($database, 'SadaPodminekPlanovani', $logDescription);
        $rec ->addField('Id', 'serial', true);
        $rec ->addStringField('Popis', false, 160, $rozdeleni ->popis . ' (' . $rozdeleni ->kod . ') - ' . StringUtils::format('date', $datum));
        $rec ->insertRecord();

        $serial = $rec ->findFieldByName('Id') ->value;

        // priradi ji rozdeleni pro aktualni den
        $rec = new RonPortalDBRecord($database, 'SadaPodminekRozdeleni', $logDescription);
        $rec ->addIntField('KodTypuRozdeleni', true, $rozdeleni ->typ);
        $rec ->addStringField('Rozdeleni', true, 30, $rozdeleni ->kod);
        $rec ->addIntField('IdSady', true, $serial);
        $rec ->addDateField('DatumOd', true, $datum);
        $rec ->addDateField('DatumDo', true, $datum);
        $rec ->addIntField('Prioritni', true, 1);
        $rec ->addIntField('TypOmezeni', true, 0);
        $rec ->insertRecord();

        return $serial;
    }


    /**
     * Vraci Id prioritni sady podminek prirazenou zadanemu rozdeleni.
     * @param Database  $database
     * @param Rozdeleni $rozdeleni
     * @param int   $datum
     * @param bool  $vytvoritNovou Pokud je true a prioritni sada neexistuje, tak ji vytvori
     * @param object|string $logDescription
     * @return int
     * @throws DBException
     */
    public static function vratIdSadyPodminekProDen($database, $rozdeleni, $datum, $vytvoritNovou, $logDescription)
    {
        $params = $database ->createParameters();
        $cursor = $database ->getSqlResult(
            'SELECT IdSady FROM SadaPodminekRozdeleni' .
            ' WHERE KodTypuRozdeleni = ' . $params ->addParameter($rozdeleni ->typ) .
            '  AND Rozdeleni = ' . $params ->addParameter($rozdeleni ->kod) .
            '  AND DatumOd = ' . $params ->addParameter($database ->dateToParam($datum)) .
            '  AND Prioritni = 1  AND  TypOmezeni = 0',
            $params
        );

        $row = $cursor ->fetch();
        $cursor ->close();

        if ($row) {
            return $row[0];
        }

        // neexistuje
        if ($vytvoritNovou) {
            return self::vytvorSaduPodminekProDen($database, $rozdeleni, $datum, $logDescription);
        }

        return null;
    }

}

==> helpdesk.ron.cz?menuid=download&filename=ronportal_4_2206_162.zip: ronportal/ronportal/kapacity/sadapodminekplanovani.php <==
<?php
/**
 * stanislav.elbl, 10.03.2021
 */

namespace RonPortal\Kapacity;

use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\StringUtils;
use RonPortal\RonPortalDBRecord;
use RonPortal\Rozdeleni;

class SadaPodminekPlanovani
{
    public $id;

    public $datumOd;
    public $datumDo;

    /** @var int    0=vychozi, 1=prioritni, 2=trvala  */
    public $prioritni;

    /** @var int    0=bez omezeni, 1=typ dne, 2=dny v tydnu */
    public $typOmezeni;
    /** @var int    typ dne podle kalendare u daneho strediska */
    public $typDne;
    /** @var int    Bitova maska - 1=Po,2=Ut,...,64=Ne */
    public $denVTydnu;

    /** @var PodminkaPlanovaniKapacit[]  Seznam podminek */
    public $podminky = [];

    /**
     * Nacte sady podminek platne pro zadane rozdeleni
     * @param Database $database
     * @param int $typRozdeleni
     * @param string $rozdeleni
     * @param int $obdobiOd
     * @param int $obdobiDo
     * @param string $filter
     * @return SadaPodminekPlanovani[]
     * @throws \Ron\DB\DBException
     */
    public static function nactiPodminky($database, $typRozdeleni, $rozdeleni, $obdobiOd, $obdobiDo, $filter = null)
    {
        $result = [];

        $params = $database ->createParameters();

        $sql =
            'SELECT SadaPodminekRozdeleni.IdSady, SadaPodminekRozdeleni.DatumOd, SadaPodminekRozdeleni.DatumDo, SadaPodminekRozdeleni.Prioritni,' . // 0..3
            ' SadaPodminekRozdeleni.TypOmezeni, SadaPodminekRozdeleni.TypDne, SadaPodminekRozdeleni.DenVTydnu,' . // 4..6
            ' PodminkaPlanovaniKapacit.Id,' .    // 7
            ' PodminkaPlanovaniKapacit.TypPodminky, PodminkaPlanovaniKapacit.Smena, PodminkaPlanovaniKapacit.CasOd, PodminkaPlanovaniKapacit.CasDo,' . // 8..11
            ' PodminkaPlanovaniKapacit.KodTypuRozdeleni, PodminkaPlanovaniKapacit.Rozdeleni,' . // 12,13
            ' PodminkaPlanovaniKapacit.KodDovednosti, PodminkaPlanovaniKapacit.KodUrovne,' . // 14,15
            ' PodminkaPlanovaniKapacit.PocetMin, PodminkaPlanovaniKapacit.PocetMax, PodminkaPlanovaniKapacit.TypKontroly,' . // 16..18
            ' PodminkaPlanovaniKapacit.SlotProPlanovani, PodminkaPlanovaniKapacit.LzeEditovat,' . // 19,20
            ' PodminkaPlanovaniKapacit.Update_UserName, PodminkaPlanovaniKapacit.Update_Timestamp' . // 21,22
            "\n" .
            'FROM SadaPodminekRozdeleni LEFT OUTER JOIN PodminkaPlanovaniKapacit' .
            '  ON PodminkaPlanovaniKapacit.IdSady = SadaPodminekRozdeleni.IdSady' .
            "\n" .
            'WHERE SadaPodminekRozdeleni.KodTypuRozdeleni = ' . $params ->addParameter($typRozdeleni) .
            '  AND SadaPodminekRozdeleni.Rozdeleni = ' . $params ->addParameter($rozdeleni) .
            '  AND SadaPodminekRozdeleni.DatumOd <= ' . $params ->addParameter($database ->dateToParam($obdobiDo)) .
            '  AND (SadaPodminekRozdeleni.DatumDo IS NULL OR SadaPodminekRozdeleni.DatumDo >= ' . $params ->addParameter($database ->dateToParam($obdobiOd)) . ')';

        if ($filter) {
            $sql .= ' AND ' . $filter;
        }

        $sql .=
            ' ORDER BY SadaPodminekRozdeleni.IdSady, SadaPodminekRozdeleni.DatumOd, PodminkaPlanovaniKapacit.CasOd';

        $cursor = $database ->getSqlResult($sql, $params);

        /** @var SadaPodminekPlanovani  $sada */
        $sada = null;

        while ($row = $cursor ->fetch()) {
            $datumOd = $database ->stringToTimestamp($row[1]);
            if (!$sada  ||  $row[0] != $sada ->id  ||  $datumOd != $sada ->datumOd) {
                $sada = new SadaPodminekPlanovani();
                $sada ->id = $row[0];
                $sada ->datumOd = $datumOd;
                $sada ->datumDo = $database ->stringToTimestamp($row[2]);
                $sada ->prioritni = (int)$row[3];
                $sada ->typOmezeni = (int)$row[4];
                $sada ->typDne = (int)$row[5];
                $sada ->denVTydnu = $row[6];
                $result[] = $sada;
            }

            if (isset($row[7])) {
                $podminka = new PodminkaPlanovaniKapacit();
                $podminka->id = (int)$row[7];
                $podminka->typPodminky = (int)$row[8];
                if ($podminka->typPodminky == 0) {
                    // smena
                    $podminka->smena = $row[9];
                }
                $podminka->casOd = $database->stringToSecondsOfDay($row[10]);
                $podminka->casDo = $database->stringToSecondsOfDay($row[11]);

                $podminka->pocetMin = (int)$row[16];
                $podminka->pocetMax = (int)$row[17];

                $podminka->kodTypuRozdeleni = ($row[12] && $row[13]) ? (int)$row[12] : null;
                $podminka->rozdeleni = ($row[12] && $row[13]) ? $row[13] : null;

                $podminka->kodDovednosti = $row[14];
                $podminka->kodUrovne = isset($row[15]) ? (int)$row[15] : null;

                $podminka->typKontroly = $row[18];

                $podminka->slotProPlanovani = $row[19];
                $podminka->lzeEditovat = $row[20];

                $podminka->updateUserName = $row[21];
                $podminka->updateTimestamp = $database ->stringToTimestamp($row[22]);

                $sada->podminky[] = $podminka;
            }
        }

        $cursor ->close();

        // TODO: pro podmiky typu "Operace dochazky" nacist prirazene operace

        return $result;
    }


    /**
     * Nacte sady podminek platne pro zadane rozdeleni a den. Vraci jen sady, ktere se maji skutecne pouzit,
     * takze pokud existuje prioritni sada, tak nevraci vychozi.
     * @param Database $database
     * @param int $typRozdeleni
     * @param string $rozdeleni
     * @param int $datum
     * @param string $filter
     * @return SadaPodminekPlanovani[]
     * @throws \Ron\DB\DBException
     */
    public static function nactiSadyPodminekProDen($database, $typRozdeleni, $rozdeleni, $datum, $filter = null)
    {
        $sadyPodminek = self::nactiPodminky($database, $typRozdeleni, $rozdeleni, $datum, $datum, $filter);

        // profiltruje jen platne sady - vynecha vychozi, pokud existuje prioritni
        $pouziteSady = [];
        $vychoziSada = null;
        $jePrioritni = false;
        foreach ($sadyPodminek as $sada) {
            if ($sada ->jePlatnaProDen($datum)) {
                if ($sada ->prioritni === 1) {
                    // prioritni
                    $pouziteSady[] = $sada;
                    $jePrioritni = true;
                } else if ($sada ->prioritni === 2) {
                    // trvala
                    $pouziteSady[] = $sada;
                } else {
                    // vychozi - pouzije se jen v pripade, ze neexistuje prioritni
                    $vychoziSada = $sada;
                }
            }
        }

        if (!$jePrioritni  &&  $vychoziSada) {
            $pouziteSady[] = $vychoziSada;
        }

        return $pouziteSady;
    }


    /**
     * @return array
     */
    public function getJson()
    {
        $result = [
            'id' => $this ->id,
            'datumOdStr' => StringUtils::jsonDate($this ->datumOd),
            'datumDoStr' => StringUtils::jsonDate($this ->datumDo),
            'prioritni' => $this ->prioritni ? 1 : 0,
            'typOmezeni' => $this ->typOmezeni,
            'typDne' => $this ->typDne,
            'denVTydnu' => $this ->denVTydnu,
            'podminky' => []
        ];

        foreach ($this ->podminky as $podminka) {
            $result['podminky'][] = $podminka ->getJson();
        }

        return $result;
    }


    /**
     * Vraci true, pokud je sada platna pro zadane datum (podle datumu platnosti a omezeni na typ dne nebo den v tydnu)
     * @param $datum
     * @param int $typDne
     * @return bool|int
     */
    public function jePlatnaProDen($datum, $typDne = 0)
    {
        // kontrola platnosti od do
        if ($datum < $this ->datumOd) {
            return false;
        }

        if ($this ->datumDo  &&  $datum > $this ->datumDo) {
            return false;
        }

        // kontrola podle typu dne
        if ($this ->typOmezeni === 1  &&  $this ->typDne != $typDne) {
            return false;
        }

        // kontrola podle dnu v tydnu
        if ($this ->typOmezeni === 2) {
            // jen vybrane dny v tydnu. Bit 1=Po atd.
            return (0x01 << (date('N', $datum) - 1)) & $this ->denVTydnu;
        }

        // zadne omezeni
        return true;
    }


    public function najdiPodminku($hledanaPodminka)
    {
        foreach ($this ->podminky as $podminka) {
            if ($podminka ->isSameAs($hledanaPodminka)) {
                return $podminka;
            }
        }

        return null;
    }


    /**
     * Vytvori novou sadu podminek a priradi ji aktualnimu rozdeleni. Vraci Id nove vytvoreneho zaznamu.
     * @param Database $database
     * @param Rozdeleni $rozdeleni
     * @param int $datum
     * @param object|string $logDescription
     * @return int
     * @throws DBException|\Exception
     */
    public static function vytvorSaduPodminekProDen($database, $rozdeleni, $datum, $logDescription)
    {
        // vytvori sadu podminek
        $rec = new RonPortalDBRecord($database, 'SadaPodminekPlanovani', $logDescription);
        $rec ->addField('Id', 'serial', true);
        $rec ->addStringField('Popis', false, 160, $rozdeleni ->popis . ' (' . $rozdeleni ->kod . ') - ' . StringUtils::format('date', $datum));
        $rec ->insertRecord();

        $serial = $rec ->findFieldByName('Id') ->value;

        // priradi ji rozdeleni pro aktualni den
        $rec = new RonPortalDBRecord($database, 'SadaPodminekRozdeleni', $logDescription);
        $rec ->addIntField('KodTypuRozdeleni', true, $rozdeleni ->typ);
        $rec ->addStringField('Rozdeleni', true, 30, $rozdeleni ->kod);
        $rec ->addIntField('IdSady', true, $serial);
        $rec ->addDateField('DatumOd', true, $datum);
        $rec ->addDateField('DatumDo', true, $datum);
        $rec ->addIntField('Prioritni', true, 1);
        $rec ->addIntField('TypOmezeni', true, 0);
        $rec ->insertRecord();

        return $serial;
    }


    /**
     * Vraci Id prioritni sady podminek prirazenou zadanemu rozdeleni.
     * @param Database  $database
     * @param Rozdeleni $rozdeleni
     * @param int   $datum
     * @param bool  $vytvoritNovou Pokud je true a prioritni sada neexistuje, tak ji vytvori
     * @param object|string $logDescription
     * @return int
     * @throws DBException
     */
    public static function vratIdSadyPodminekProDen($database, $rozdeleni, $datum, $vytvoritNovou, $logDescription)
    {
        $params = $database ->createParameters();
        $cursor = $database ->getSqlResult(
            'SELECT IdSady FROM SadaPodminekRozdeleni' .
            ' WHERE KodTypuRozdeleni = ' . $params ->addParameter($rozdeleni ->typ) .
            '  AND Rozdeleni = ' . $params ->addParameter($rozdeleni ->kod) .
            '  AND DatumOd = ' . $params ->addParameter($database ->dateToParam($datum)) .
            '  AND Prioritni = 1  AND  TypOmezeni = 0',
            $params
        );

        $row = $cursor ->fetch();
        $cursor ->close();

        if ($row) {
            return $row[0];
        }

        // neexistuje
        if ($vytvoritNovou) {
            return self::vytvorSaduPodminekProDen($database, $rozdeleni, $datum, $logDescription);
        }

        return null;
    }

}
